/*program to calculate posterior features, generalized impulse response
functions and forecasts  for 2 regime tar models with delays
based on the
 (i) previous average differences of the time series and regime specific variance

need to load data assumed to be placed in vector u  program
calculates the marginal likelihood for a fixed value of
p that user needs to specify from the results of using 2taru.prg


First version March 12, 1997
Simon Potter,


This version February 10  1998
minor revisions july 22, 1998
*/
new;

clear p, ceout, giout;

#include gi2tara.prc;
#include for2tara.prc;
#include coefftab;
#include gitab;
prtime=hsec;
/******************Choose the lag length********************
*********must be bigger than 0**************************/
_psel=5;

/* check the sample size used here and go to line approx 282 to set up
simulation parameters*/


load hgnp;
u=100*ln(hgnp[2:rows(hgnp),.]./hgnp[1:rows(hgnp)-1,.]);
sampm=meanc(u);
dampm=sortc(abs(u[2:rows(u),.]-u[1:rows(u)-1,.]),1);
dampm=meanc(dampm);
 /*this will determine average rate of change for initial condition in gi analysis*/
u=u-meanc(u);
pmax=5;  /*keep this the same as in model search section*/

dmax=_psel;
pdmax=maxc(pmax|dmax)+1; /*differences will use one more observation*/

t=rows(u);
y=u[pdmax+1:t,.];
xdata=y~ones(t-pdmax,1);
for j (1,pdmax,1);
xdata=xdata~u[pdmax+1-j:t-j,.];
endfor;
t=rows(xdata);
/*Parameters of the prior*/
rl=round(0.15*t); /*smallest number in lower regime and lowest s
start point for upper regime*/
 /*lowest start point for upper regime*/
ru=round(0.85*t); /*highest end point  for lower regime
and highest start point for upper regime*/

/*check prior parameters are the same as in main program*/
_v=3;  /*degrees of freedom for the variance*/
_v_s=1; /*mean of the variance*/
_c=(_v_s)^(_v/2)/gamma(_v/2);
c=_c^2;


vbar=_v+t;

lgvbar=lnfact(vbar/2-1); /*posterior degrees of freedom*/

/*storage for posterior on r1,r2,d*/
_rpost=zeros(1,5);
/*storage for posterior means etc*/
_mpost=zeros(1,2+1);
_spost=zeros(1,2+1);
_vpost=zeros(1,4);
_qpost=zeros(1,2*(_psel+1)+1);

_lsee=zeros(1,1); /*finds the least squares maximizer*/



p=_psel;
/*************************
Parameters of the prior  for the regime coefts
*******************************************************/
_hypec=4;   /* variance for the constant*/
_hyped=1;  /* variance for the first ar lag coeff't*/
_hypes=0.8; /*shrinkage factor*/

_d=(_hypec|_hyped*seqm(1,_hypes,p))/(_v_s/(_v-2)); /*prior for the regression coefficients*/

_hd=diagrv(zeros(2*(p+1),2*(p+1)),_d|_d);
_d=diagrv(zeros(p+1,p+1),_d);

ipq=invpd(_d);
ipq1=ipq;
ipq2=ipq;
dipq=sqrt(det(ipq));
dipq1=dipq;
dipq2=dipq;

_hipq=invpd(_hd);
_hdipq=sqrt(det(_hipq));
for d (2,p,1);

_bbigx=sortc(xdata[.,1:p+2]~(xdata[.,3]-xdata[.,d+3]), p+2+1);


/*storage for threshold and threshold gaps */

/*now loop to get all the different marginal likelihoods*/


for i (rl,ru,1);
/*storage for intermediate results*/

if i==rl;


_by1=_bbigx[1:rl,1];
_bx1=_bbigx[1:rl,2:p+2];
_bq1=moment(_bx1,0);
_bbq1=invpd(ipq1+_bq1);
_bqq1=invpd(_bq1);
_bb1=_bqq1*_bx1'_by1; /*least squares estimator*/
_bsee1=(_by1-_bx1*_bb1)'(_by1-_bx1*_bb1);

_bbb1=_bbq1*_bx1'_by1; /*posterior mean estimator*/
_bvbar1=_v+rows(_by1); /*degrees of freedom for the variance*/


else;
/*use the add one in estimator to recursively update the required quanitities*/
_by1=_by1|_bbigx[i,1];
_bx1=_bx1|_bbigx[i,2:p+2];
_bvbar1=_bvbar1+1;
x=_bbigx[i,2:p+2];
xx=moment(x,0);
fq=x*_bqq1*x'+1;
fqb=x*_bbq1*x'+1;
_bb1=_bb1+_bqq1*x'*(_by1[i,1]-x*_bb1)/fq;
_bq1=_bq1+xx;
_bqq1=_bqq1-_bqq1*xx*_bqq1/(fq);
_bsee1=(_by1-_bx1*_bb1)'(_by1-_bx1*_bb1);
_bbb1=_bbb1+_bbq1*x'*(_by1[i,1]-x*_bbb1)/fqb;
_bbq1=_bbq1-_bbq1*xx*_bbq1/(fqb);

endif;

_r1=_bbigx[i,cols(_bbigx)];
_br1=_bbigx[i+1,cols(_bbigx)]-_bbigx[i,cols(_bbigx)];
_bvsbar1=_v_s+_bsee1+(_bbb1-_bb1)'_bq1*(_bbb1-_bb1)+(_bbb1'ipq1*_bbb1);
_bd1=sqrt(det(_bbq1));
_bml1=gamma(_bvbar1/2)*_bd1*dipq1*(_bvsbar1^(-_bvbar1/2));

if i==rl;
_by2=_bbigx[1+i:t,1];  /*upper regime*/
_bx2=_bbigx[1+i:t,2:p+2];





_bq2=moment(_bx2,0);
_bbq2=invpd(ipq2+_bq2);
_bqq2=invpd(_bq2);
_bb2=_bqq2*_bx2'_by2; /*least squares estimator*/
_bsee2=(_by2-_bx2*_bb2)'(_by2-_bx2*_bb2);

_bbb2=_bbq2*_bx2'_by2; /*posterior mean estimator*/
_bvbar2=_v+rows(_by2); /*degrees of freedom for the variance*/





else;

_by2=_bbigx[i+1:t,1];
_bx2=_bbigx[i+1:t,2:p+2];

_bvbar2=_bvbar2-1;

/*take away one estimator for upper regime*/
x=_bbigx[i,2:p+2];
y=_bbigx[i,1];
xx=moment(x,0);
fq=x*_bqq2*x'-1;
fqb=x*_bbq2*x'-1;
_bb2=_bb2+_bqq2*x'*(y-x*_bb2)/fq;
_bsee2=(_by2-_bx2*_bb2)'(_by2-_bx2*_bb2);
_bbb2=_bbb2+_bbq2*x'*(y-x*_bbb2)/fqb;
_bq2=_bq2-xx;
_bqq2=_bqq2-_bqq2*xx*_bqq2/(fq);
_bbq2=_bbq2-_bbq2*xx*_bbq2/(fqb);


endif;





_bvsbar2=_v_s+_bsee2+(_bbb2-_bb2)'_bq2*(_bbb2-_bb2)+(_bbb2'ipq2*_bbb2);
_bd2=sqrt(det(_bbq2));
_bml2=gamma(_bvbar2/2)*_bd2*dipq2*(_bvsbar2^(-_bvbar2/2));

_bml123=(_bml1*_bml2);







_mpost=_mpost|((_bbb1~_bbb2)~
(ones(p+1,1)*_rpost[rows(_rpost),cols(_rpost)]+1));

_spost=_spost|((_bvsbar1/(_bvbar1-2)*diag(_bbq1))
~(_bvsbar2/(_bvbar2-2)*diag(_bbq2))~
(ones(p+1,1)*_rpost[rows(_rpost),cols(_rpost)]+1));
_vpost=_vpost|(_bvsbar1~_bvbar1~_bvsbar2~_bvbar2);
_qpost=_qpost|(chol(_bbq1)~chol(_bbq2)~
(ones(p+1,1)*_rpost[rows(_rpost),cols(_rpost)]+1));

_rpost=_rpost|(_bml123~d~_r1~_br1
~(_rpost[rows(_rpost),cols(_rpost)]+1));


_lsee=_lsee|(_bsee1+_bsee2);







endfor;

endfor;


_rpost=_rpost[2:rows(_rpost),.];
_mpost=_mpost[2:rows(_mpost),.];
_spost=_spost[2:rows(_spost),.];
_qpost=_qpost[2:rows(_qpost),.];

_vpost=_vpost[2:rows(_vpost),.];

_lsee=_lsee[2:rows(_lsee),.];

/*first find average features of the posterior*/
_rpost[.,1]=(_rpost[.,1])/sumc(_rpost[.,1]).*_rpost[.,4];
dpost=zeros(dmax-1,1);
for d (2,p,1);
dpost[d-1,.]=sumc(selif(_rpost[.,1],_rpost[.,2].==d));

endfor;
r1post=_rpost[.,3]'_rpost[.,1]; /* mean value of threshold 1*/

/*average the ar coeff'ts*/

mb1=reshape(_mpost[.,1],rows(_rpost),p+1)'_rpost[.,1];
mb2=reshape(_mpost[.,2],rows(_rpost),p+1)'_rpost[.,1];
/*average the posterior variance of the coeff'ts*/
sb1=sqrt((reshape(_spost[.,1],rows(_rpost),p+1))'_rpost[.,1]+
((reshape(_mpost[.,1],rows(_rpost),p+1)-mb1')^2)'_rpost[.,1]);

sb2=sqrt((reshape(_spost[.,2],rows(_rpost),p+1))'_rpost[.,1]+
((reshape(_mpost[.,2],rows(_rpost),p+1)-mb2')^2)'_rpost[.,1]);

/*average the regime innovation variances*/
vs1=(_vpost[.,1]./(_vpost[.,2]-2))'_rpost[.,1];
vs2=(_vpost[.,3]./(_vpost[.,4]-2))'_rpost[.,1];





/*now convert to cdf*/
_rpost[.,1]=cumsumc(_rpost[.,1]);



/***********************************************************
now loop to generate dynamic features of the model, need to choose simulation parameters
***********************************************************/



kk=10; /*number of draws from the posterior of the threshold parameters*/
nnf=8;  /*horizon for the forecast*/
nngi=20; /*horizon for the gi*/
gi=zeros(nngi+1,6);
_for=zeros(nnf,2);

/* need to decide the dynamic property of interest, then need
to go to line 345 to set up output file name*/
k=10; /*number of draws of regime coeeft's per threshold draw*/
ngi=100;  /*number of draws of future shocks*/



for i (1, kk,1);
/*first draw d,r1*/
ii=maxindc(_rpost[.,1].>rndu(1,1));
dd=_rpost[ii,2];
r1=_rpost[ii,3]+_rpost[ii,4]*rndu(1,1);
sb=selif(_mpost[.,1:cols(_mpost)-1],_mpost[.,cols(_mpost)].==ii);
sq=selif(_qpost[.,1:cols(_qpost)-1],_qpost[.,cols(_qpost)].==ii);
ss=_vpost[ii,.];
/*now draw regime cofficients  */

s1=sumc((rndn(ss[.,2],k)^2)/ss[.,1])';
b1=sb[.,1]+sqrt(s1^(-.5)).*(sq[.,1:1*(p+1)]*rndn(p+1,k));
s2=sumc((rndn(ss[.,4],k)^2)/ss[.,3])';

b2=sb[.,2]+sqrt(s2^(-.5)).*(sq[.,(p+1)+1:2*(p+1)]*rndn(p+1,k));



/*now draw the generalized impulses and/or forecasts
main choice is the initial conditions*/

gi[.,1:2]=gi[.,1:2] + gi2tara(b1,b2,r1,dd,
s1^(-0.5),s2^(-0.5), seqa(0,-dampm/(p+1),p+1) ,k,ngi, 2, -2,nngi)/kk;
gi[.,3:4]=gi[.,3:4] + gi2tara(b1,b2,r1,dd,
s1^(-0.5),s2^(-0.5), seqa(0,dampm/(p+1),p+1) ,k,ngi, 2, -2,nngi)/kk;
gi[.,5:6]=gi[.,5:6] + gi2tara(b1,b2,r1,dd,
s1^(-0.5),s2^(-0.5), zeros(p+1,1) ,k,ngi, 2, -2,nngi)/kk;
_for=_for + for2tara(b1,b2,r1,dd,
s1^(-0.5),s2^(-0.5),u[rows(u)-p:rows(u),.],k,ngi,nnf)/kk;




endfor;
/*now find the least squares estimator*/
ii=minindc(_lsee);
dols=_rpost[ii,2];
r1ols=_rpost[ii,3];
_abigx=sortc(xdata[.,1:p+2]~(xdata[.,3]-xdata[.,dols+3]), p+2+1);

e1= (_abigx[.,cols(_abigx)].<=r1ols);
e0=1-e1;
_ay1=selif(_abigx[.,1], e1);
_ax1=selif(_abigx[.,2:p+2], e1);
_ay2=selif(_abigx[.,1],e0);
_ax2=selif(_abigx[.,2:p+2],e0);
xx1ols=invpd(moment(_ax1,0));
xx2ols=invpd(moment(_ax2,0));
_b1ols=xx1ols*_ax1'_ay1;
_b2ols=xx2ols*_ax2'_ay2;
s1ols=(_ay1-_ax1*_b1ols)'(_ay1-_ax1*_b1ols)/rows(_ay1);
s2ols=(_ay2-_ax2*_b2ols)'(_ay2-_ax2*_b2ols)/rows(_ay2);
sb1ols=sqrt(diag(s1ols*xx1ols));
sb2ols=sqrt(diag(s2ols*xx2ols));


ceft=(mb1|vs1|mb2|vs2|r1post)~(sb1|0|sb2|0|0)~
(_b1ols|s1ols|_b2ols|s2ols|r1ols)~(sb1ols|0|sb2ols|0|0);
ceout="2tara" $+ftocv(p,2,0) $+ ".out";
giout="2gia" $+ftocv(p,2,0) $+ ".out";

/*coefftab is the keyword, ceft is the matrix with coefft's,
 the output file name automatically change for different choices of p*/
coefftab ceft;
gitab gi;
gisave="gi2a"$+ftocv(p,2,0);
forsave="for2a"$+ftocv(p,2,0);
save ^gisave=gi, ^forsave=_for;
/***************
now construct graphs ******************************
set up for pc dos not unix or windows gauss yet   ********************/
/*library pgraph;
graphset;
_pdate=0;
xlabel("Horizon(n)");
ylabel("Normalized GI");
_plegctl=1;
_plegstr="Fast Decrease\000Fast Increase\000No Change";
title("Positive Shock");
cmdstr="-p -pf=pos2a" $+ftocv(p,2,0)$+".eps";
graphprt(cmdstr);
xy(seqa(0,1,nngi+1),gi[.,1|3|5]./gi[1,1|3|5]);
title("Negative Shock");
cmdstr="-p -pf=neg2a" $+ftocv(p,2,0)$+".eps";
graphprt(cmdstr);
xy(seqa(0,1,nngi+1),-gi[.,2|4|6]./gi[1,2|4|6]);
_plegstr="Positive Shock\000Negative Shock";
ylabel("Absolute Value of GI");
title("Fast Decrease");
cmdstr="-p -pf=dec2a" $+ftocv(p,2,0)$+".eps";
graphprt(cmdstr);

xy(seqa(0,1,nngi+1),abs(gi[.,3:4]));
title("Fast Increase");
cmdstr="-p -pf=inc2a" $+ftocv(p,2,0)$+".eps";
graphprt(cmdstr);

xy(seqa(0,1,nngi+1),abs(gi[.,1:2]));
title("No Change");
cmdstr="-p -pf=noc2a" $+ftocv(p,2,0)$+".eps";
graphprt(cmdstr);
xy(seqa(0,1,nngi+1),abs(gi[.,5:6]));

sols=minc(_lsee)/t;
lsee=selif(_lsee,_rpost[.,2].==dols);
rols=selif(_rpost[.,3],_rpost[.,2].==dols);
rpost=selif(_rpost[.,1],_rpost[.,2].==dols);
k=rows(lsee);
cci=sols*(ones(k,1)+7.35/t);
title("Classical Confidence Interval for Threshold");
xlabel("Threshold Values");
ylabel("Residual Variance");
_plegstr="Residual Variance\00095%confid line";
xy(rols, (lsee/t)~cci);
title("Bayesian Posterior for r conditional on mle of d");
ylabel("Posterior Probability");
_plegstr="Probability of r";
xy(rols, rpost);*/


"time to run in minutes " (hsec-prtime)/6000;


